package com.yineng.ynbridge

import android.text.TextUtils

/**
 * ClassName PluginManager
 * User: zuoweichen
 * Date: 2022/3/17 13:43
 * Description: 插件管理器
 */
class PluginManager {
    private val pluginMap = linkedMapOf<String, YnWebPlugin?>()
    private val entryMap = linkedMapOf<String, PluginEntry>()
    private val app: YnBridgeWebView
    private var isInitialized = false
    private val btc: YnBridgeInterface

    constructor(
        app: YnBridgeWebView,
        bridgeInterface: YnBridgeInterface,
        pluginEntries: MutableList<PluginEntry>
    ) {
        this.app = app
        this.btc = bridgeInterface
        setPluginEntries(pluginEntries)
    }

    fun setPluginEntries(pluginEntries: Collection<PluginEntry>) {
        if (isInitialized) {

            pluginMap.clear()
            entryMap.clear()
        }
        for (entry in pluginEntries) {
            addService(entry)
        }
        if (isInitialized) {
            startupPlugins()
        }
    }

    fun init() {
        isInitialized = true
        pluginMap.clear()
    }

    /**
     * 创建具有onLoad设置的插件对象。
     */
    private fun startupPlugins() {
        for (entry in entryMap.values) {
            // Add a null entry to for each non-startup plugin to avoid ConcurrentModificationException
            // When iterating plugins.
            if (entry.onLoad) {
                getPlugin(entry.service)
            } else {
                pluginMap[entry.service] = null
            }
        }
    }

    /**
     * 接收一个执行请求，并通过找到适当的Java类并调用它的execute方法来实现它。
     * PluginManager。Exec既可以同步使用也可以异步使用。在这两种情况下，都会返回一个JSON编码的字符串，
     * 该字符串将指示在试图查找或执行clazz参数所表示的类时是否发生了错误。
     * @param service String?
     * @param action String?
     * @param callbackId String?
     * @param arguments String?
     */
    fun exec(
        service: String,
        action: String,
        callbackId: String?,
        rawArgs: String
    ) {
        val plugin = getPlugin(service)
        if (plugin == null) {
            return
        }
        val callbackContext = CallbackContext(callbackId, app)
        val wasValidAction = plugin.execute(action, rawArgs, callbackContext)
        if (!wasValidAction) {
            //TODO something
        }
    }

    fun getPlugin(service: String): YnWebPlugin? {
        var ret = pluginMap[service]
        if (ret == null) {
            var pe: PluginEntry? = entryMap[service]
            if (pe == null) {
                return null
            }
            if (pe.plugin != null) {
                ret = pe.plugin
            } else {
                ret = instantiatePlugin(pe.pluginClass)
            }
            if (ret != null) {
                ret.privateInitialize(service, app, btc)
                pluginMap[service] = ret
            }
        }
        return ret
    }

    /**
     *  在服务条目表中添加一个实现服务的插件类。这不会创建插件对象实例。
     * @param entry PluginEntry
     */
    fun addService(entry: PluginEntry) {
        entryMap[entry.service] = entry
        if (entry.plugin != null) {
            entry.plugin.privateInitialize(entry.service, app,btc)
            pluginMap[entry.service] = entry.plugin
        }
    }

    fun addService(service: String, className: String) {
        this.addService(PluginEntry(service, className))
    }

    private fun instantiatePlugin(className: String): YnWebPlugin? {
        var ret: YnWebPlugin? = null
        try {
            if (!TextUtils.isEmpty(className)) {
                val cl = Class.forName(className)
                if (YnWebPlugin::class.java.isAssignableFrom(cl)) {
                    ret = cl.newInstance() as YnWebPlugin
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
            println("Error adding plugin $className.")
        }
        return ret
    }

    fun postMessage(id: String, data: Any?) {
        btc.onMessage(id, data)
    }
}